import { ArgumentInvalidException, ValueObject } from '@mobicoop/ddd-library';
import { Role } from '../ad.types';
import { Target } from '../candidate.types';
import { ActorProps } from './actor.value-object';

/** Note:
 * Value Objects with multiple properties can contain
 * other Value Objects inside if needed.
 * */

export interface ActorTimeProps extends ActorProps {
  time: string;
  minTime: string;
  maxTime: string;
}

export class ActorTime extends ValueObject<ActorTimeProps> {
  get time(): string {
    return this.props.time;
  }

  get minTime(): string {
    return this.props.minTime;
  }

  get maxTime(): string {
    return this.props.maxTime;
  }
  get role(): Role {
    return this.props.role;
  }

  get target(): Target {
    return this.props.target;
  }

  protected validate(props: ActorTimeProps): void {
    this._validateTime(props.time, 'time');
    this._validateTime(props.minTime, 'minTime');
    this._validateTime(props.maxTime, 'maxTime');
  }

  private _validateTime(time: string, property: string): void {
    if (time.split(':').length != 2)
      throw new ArgumentInvalidException(`${property} is invalid`);
    if (parseInt(time.split(':')[0]) < 0 || parseInt(time.split(':')[0]) > 23)
      throw new ArgumentInvalidException(`${property} is invalid`);
    if (parseInt(time.split(':')[1]) < 0 || parseInt(time.split(':')[1]) > 59)
      throw new ArgumentInvalidException(`${property} is invalid`);
  }
}
