import {
  ArgumentOutOfRangeException,
  ValueObject,
} from '@mobicoop/ddd-library';
import { ScheduleItemProps } from './schedule-item.value-object';
import { ActorTime } from './actor-time.value-object';

/** Note:
 * Value Objects with multiple properties can contain
 * other Value Objects inside if needed.
 * */

export interface JourneyProps extends ScheduleItemProps {
  firstDate: Date;
  lastDate: Date;
  actorTimes: ActorTime[];
}

export class Journey extends ValueObject<JourneyProps> {
  get firstDate(): Date {
    return this.props.firstDate;
  }

  get lastDate(): Date {
    return this.props.lastDate;
  }

  get actorTimes(): ActorTime[] {
    return this.props.actorTimes;
  }

  protected validate(props: JourneyProps): void {
    if (props.firstDate > props.lastDate)
      throw new ArgumentOutOfRangeException(
        'firstDate must be before lastDate',
      );
    if (props.actorTimes.length <= 0)
      throw new ArgumentOutOfRangeException(
        'at least one actorTime is required',
      );
  }
}
